from XenGarden.Console import Console
from XenGarden.GuestMetrics import GuestMetrics
from XenGarden.Host import Host
from XenGarden.SR import SR
from XenGarden.VBD import VBD
from XenGarden.VDI import VDI
from XenGarden.VIF import VIF
from XenGarden.VM import VM

__all__ = [
    "SR",
    "VBD",
    "VDI",
    "VIF",
    "Console",
    "GuestMetrics",
    "Host",
    "VM",
    "session",
]
__version__ = "1.2.0"
__copyright__ = "Copyright (c) 2020-2021 Stella IT Inc."



import XenAPI


def create_session(_id, get_xen_clusters: dict):
    xen_clusters = get_xen_clusters
    cred = xen_clusters[_id]
    session = XenAPI.Session(cred["host"])
    session.xenapi.login_with_password(cred["username"], cred["password"])
    return session



from deprecated import deprecated
from XenAPI.XenAPI import Failure

from XenGarden.VDI import VDI
from XenGarden.VM import VM


class VBD:
    """ The Virtual Block Device Object """

    def __init__(self, session, vbd):
        self.session = session
        self.vbd = vbd

    @staticmethod
    def get_by_uuid(session, uuid):
        """ returns SR object that has specific uuid """

        vbd = session.xenapi.VBD.get_by_uuid(uuid)

        if vbd is not None:
            return VBD(session, vbd)
        else:
            return None

    @staticmethod
    def get_all(session):
        """ returns VBDs existing on this _host """

        vbds = session.xenapi.VBD.get_all()
        vbd_list = []

        for vbd in vbds:
            vbd_list.append(VBD(session, vbd))

        return vbd_list

    @staticmethod
    def create(
        session,
        vm: VM,
        vdi: VDI,
        userdevice="0",
        bootable=True,
        mode="RW",
        disk_type="Disk",
        empty=False,
        qos_algorithm_type="",
        qos_algorithm_params={},
        other_config={},
        **kwargs
    ):
        _vm = vm.vm
        _vdi = vdi.vdi

        _vbd = session.xenapi.VBD.create(
            {
                "VM": _vm,
                "VDI": _vdi,
                "userdevice": userdevice,
                "bootable": bootable,
                "mode": mode,
                "type": disk_type,
                "empty": empty,
                "qos_algorithm_type": qos_algorithm_type,
                "qos_algorithm_params": qos_algorithm_params,
                "other_config": other_config,
                **kwargs,
            }
        )

        return VBD(session, _vbd)

    @deprecated
    def serialize(self) -> dict:
        vm = self.get_VM()
        vdi = self.get_VDI()

        if vm is not None:
            vm = vm.serialize()

        if vdi is not None:
            vdi = vdi.serialize()

        return {
            "_vm": vm,
            "vdi": vdi,
            "bootable": self.get_bootable(),
            "attached": self.get_currently_attached(),
            "unpluggable": self.get_unpluggable(),
            "device": self.get_device(),
            "type": self.get_type(),
            "uuid": self.get_uuid(),
            "mode": self.get_mode(),
        }

    def get_VM(self):
        """ get VM attached to the specified VBD """

        try:
            from XenGarden.VM import VM

            vm = self.session.xenapi.VBD.get_VM(self.vbd)
            vm = VM(self.session, vm)

            vm.get_uuid()
            return vm
        except Failure as xenapi_error:
            if xenapi_error.details[0] == "HANDLE_INVALID":
                return None
            else:
                raise xenapi_error

    def get_VDI(self) -> VDI:
        """ get VDI attached to the specified VBD """
        try:
            vdi = self.session.xenapi.VBD.get_VDI(self.vbd)
            vdi = VDI(self.session, vdi)
            vdi.get_uuid()

            return vdi
        except Failure as xenapi_error:
            if xenapi_error.details[0] == "HANDLE_INVALID":
                return None
            else:
                raise xenapi_error

    def get_record(self):
        """ Returns Information of the VM """
        return self.session.xenapi.VBD.get_record(self.vbd)

    def get_uuid(self) -> str:
        """ get UUID of VBD """

        return self.session.xenapi.VBD.get_uuid(self.vbd)

    def get_mode(self) -> str:
        """ get mode of VBD """

        return self.session.xenapi.VBD.get_mode(self.vbd)

    def get_type(self) -> str:
        """ get type of VBD """

        return self.session.xenapi.VBD.get_type(self.vbd)

    def get_bootable(self) -> bool:
        """ get VBD is bootable """

        return self.session.xenapi.VBD.get_bootable(self.vbd)

    def get_currently_attached(self) -> bool:
        """ get VBD is currently attached """

        return self.session.xenapi.VBD.get_currently_attached(self.vbd)

    def get_unpluggable(self) -> bool:
        """ get VBD is unpluggable """

        return self.session.xenapi.VBD.get_unpluggable(self.vbd)

    def get_device(self) -> str:
        """ get VBD device """

        return self.session.xenapi.VBD.get_device(self.vbd)

    def destroy(self) -> bool:
        """ eject VDI from VBD """

        self.session.xenapi.VBD.destroy(self.vbd)
        return True

    def insert(self, vdi: VDI) -> bool:
        """ insert VDI to VBD """

        self.session.xenapi.VBD.insert(self.vbd, vdi.vdi)
        return True

    def eject(self) -> bool:
        """ eject VDI from VBD """

        self.session.xenapi.VBD.eject(self.vbd)
        return True

    def plug(self) -> bool:
        """ plug specified VBD """

        self.session.xenapi.VBD.plug(self.vbd)
        return True

    def unplug(self) -> bool:
        """ unplug VBD """

        self.session.xenapi.VBD.unplug(self.vbd)
        return True

    def unplug_force(self) -> bool:
        """ unplug VBD """

        self.session.xenapi.VBD.unplug_force(self.vbd)
        return True



import asyncio

import XenAPI


class XenGardenAsyncTimeoutException(Exception):
    """Exception raised for timeout during processing xenapi query

    Attributes:
        session -- the session where exception has occurred
        task -- the task which caused timeout
    """

    def __init__(self, session, task, message="XenAPI Timeout occurred in XenGarden"):
        self.session = session
        self.task = task
        super().__init__(self.message)


class Common:

    TIMEOUT = 5000

    @classmethod
    async def xenapi_task_handler(
        cls, session, task, ignore_timeout=False, show_progress=False
    ):
        """A pseudo-xenapi asyncio-ifier via implementing event loop by myself."""
        cycle_passed = 0

        while session.xenapi.task.get_status(task) == "pending" and (
            cycle_passed <= cls.TIMEOUT or ignore_timeout
        ):

            await asyncio.sleep(1)

            if show_progress:
                progress = round(session.xenapi.task.get_progress(task), 2) * 100
                print(str(progress) + "% Complete!", flush=True)

            cycle_passed += 1

        if cycle_passed > cls.TIMEOUT:
            raise XenGardenAsyncTimeoutException()

        session.xenapi.task.get_record(task)
        result = session.xenapi.task.get_result(task)
        error = session.xenapi.task.get_error_info(task)

        result = result.replace("<value>", "")
        result = result.replace("</value>", "")

        if len(error) > 0:
            raise XenAPI.Failure(error)

        return result



from deprecated import deprecated


class GuestMetrics:
    def __init__(self, session, guest):
        self.session = session
        self.guest = guest

    @staticmethod
    def get_by_uuid(session, uuid):
        """ returns GuestMetrics object that has specific uuid """

        guest_metrics = session.xenapi.VM_guest_metrics.get_by_uuid(uuid)
        if guest_metrics is not None:
            return GuestMetrics(session, guest_metrics)
        else:
            return None

    def get_record(self):
        return self.session.xenapi.VM.get_record(self.guest)

    @deprecated
    def serialize(self):
        return {
            "uuid": self.get_uuid(),
            "os": self.get_os_version(),
            "networks": self.get_networks(),
        }

    def get_record(self):
        """ Returns Information of the Guest """
        return self.session.xenapi.VM_guest_metrics.get_record(self.guest)

    def get_uuid(self):
        """ Gets UUID of Guest Metrics """

        return self.session.xenapi.VM_guest_metrics.get_uuid(self.guest)

    def get_networks(self):
        """ Gets UUID of Guest Networks """

        return self.session.xenapi.VM_guest_metrics.get_networks(self.guest)

    def get_os_version(self):
        """ Gets OS Version of Guest """

        return self.session.xenapi.VM_guest_metrics.get_os_version(self.guest)

    def get_other(self):
        """ Gets "Other" of Guest """

        return self.session.xenapi.VM_guest_metrics.get_other(self.guest)



import math

from XenAPI.XenAPI import Failure

from XenGarden.Common import Common
from XenGarden.Console import Console
from XenGarden.GuestMetrics import GuestMetrics
from XenGarden.SR import SR


class VM:
    """ The Virtual Machine Object """

    def __init__(self, session, vm):
        self.session = session
        self.vm = vm

    @staticmethod
    def get_by_uuid(session, uuid):
        """ returns VM object that has specific uuid """

        vm = session.xenapi.VM.get_by_uuid(uuid)

        if vm is not None:
            return VM(session, vm)
        else:
            return None

    @staticmethod
    def list_templates(session):
        """gets Templates available in specific XenServer
        session: the XenServer Connection Session
        Returns Available Templates (List of VM object)"""

        allTemplates = []
        allVMs = session.xenapi.VM.get_all_records()

        for vmF in allVMs:
            template = VM(session, vmF)

            if template.is_template():
                allTemplates.append(template)

        return allTemplates

    @staticmethod
    def list_vm(session):
        """gets VMs available in specific XenServer
        session: the XenServer Connection Session
        Returns Available Templates (List of VM object)"""

        allVMList = []
        allVMs = session.xenapi.VM.get_all()

        for vmF in allVMs:
            vm = VM(session, vmF)
            if not vm.is_template():
                allVMList.append(vm)

        return allVMList

    def get_uuid(self):
        """ get UUID of VM """

        return self.session.xenapi.VM.get_uuid(self.vm)

    def is_template(self):
        """ Returns rather this _vm is template """

        return self.session.xenapi.VM.get_is_a_template(self.vm)

    def get_record(self):
        """ Returns Information of the VM """
        return self.session.xenapi.VM.get_record(self.vm)

    def get_power_state(self):
        """ Returns Power State of the VM """

        return self.session.xenapi.VM.get_power_state(self.vm)

    def get_consoles(self):
        """ Returns Consoles of the VM """

        consoles = self.session.xenapi.VM.get_consoles(self.vm)

        consoleList = []
        for console in consoles:
            consoleList.append(Console(self.session, console))
        return consoleList

    def get_guest_metrics(self):
        """ Returns Guest Metrics Object of the VM """
        try:
            guest_metrics = self.session.xenapi.VM.get_guest_metrics(self.vm)
            guest_metrics = GuestMetrics(self.session, guest_metrics)
            guest_metrics.get_uuid()

            return guest_metrics
        except Failure as xenapi_error:
            if xenapi_error.details[0] == "HANDLE_INVALID":
                return None
            else:
                raise xenapi_error

    def get_snapshots(self):
        """ Returns Available Snapshots (List of VM object) """

        allSnapshots = []
        snapshots = self.session.xenapi.VM.get_snapshots(self.vm)

        for snapshotNo in snapshots:
            try:
                vm = VM(self.session, snapshots[snapshotNo])
                vm.uuid()

                allSnapshots.append(vm)
            except:
                pass

        return allSnapshots

    def get_name(self):
        """ Returns VM's XenServer Name """

        return self.session.xenapi.VM.get_name_label(self.vm)
        #
        return True

    def get_description(self):
        """ Returns VM's XenServer Description """

        return self.session.xenapi.VM.get_name_description(self.vm)
        #
        return True

    async def start(self):
        """ Starts VM (Returns Boolean, True: Success, False: Fail) """

        task = self.session.xenapi.Async.VM.start(self.vm, False, False)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def shutdown(self):
        """ Shutdowns VM (Returns Boolean, True: Success, False: Fail) """

        task = self.session.xenapi.Async.VM.clean_shutdown(self.vm)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def force_shutdown(self):
        """ Force Shutdown VM (Returns Boolean, True: Success, False: Fail) """

        task = self.session.xenapi.Async.VM.hard_shutdown(self.vm)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def reboot(self):
        """ Reboot VM (Returns Boolean, True: Success, False: Fail) """

        task = self.session.xenapi.Async.VM.clean_reboot(self.vm)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def force_reboot(self):
        """ Force Reboot VM (Returns Boolean, True: Success, False: Fail) """

        task = self.session.xenapi.Async.VM.hard_reboot(self.vm)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def snapshot(self, snapshot_name):
        """ Take a snapshot of current VM """

        task = self.session.xenapi.Async.VM.snapshot(self.vm, snapshot_name)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def suspend(self):
        """ Suspend VM """
        task = self.session.xenapi.Async.VM.suspend(self.vm)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def resume(self):
        task = self.session.xenapi.VM.resume(self.vm, True, False)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def pause(self):
        task = self.session.xenapi.Async.VM.pause(self.vm)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def unpause(self):
        task = self.session.xenapi.Async.VM.unpause(self.vm)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def provision(self):
        task = self.session.xenapi.Async.VM.provision(self.vm)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def clone(self, new_name):
        task = self.session.xenapi.Async.VM.clone(self.vm, new_name)
        data = await Common.xenapi_task_handler(self.session, task, True)
        vm = VM(self.session, data)

        return vm

    async def copy(self, new_name, sr: SR):
        task = self.session.xenapi.Async.VM.copy(self.vm, new_name, sr.sr)
        data = await Common.xenapi_task_handler(self.session, task, True)
        vm = VM(self.session, data)

        return vm

    def set_name(self, name):
        self.session.xenapi.VM.set_name_label(self.vm, name)
        return True

    def set_description(self, description):
        self.session.xenapi.VM.set_name_description(self.vm, description)
        return True

    def set_vCPUs(self, vCPUs, sockets=1):
        tmp_platform = self.session.xenapi.VM.get_platform(
            self.vm,
        )
        vCPU_platform = {"cores-per-socket": str(math.floor(vCPUs / sockets))}

        platform = {**tmp_platform, **vCPU_platform}

        self.session.xenapi.VM.set_platform(self.vm, platform)
        self.session.xenapi.VM.set_VCPUs_max(self.vm, vCPUs)
        self.session.xenapi.VM.set_VCPUs_at_startup(self.vm, vCPUs)

        return True

    def set_memory(self, memory):
        self.session.xenapi.VM.set_memory_limits(
            self.vm, memory, memory, memory, memory
        )
        return True

    def get_platform(self):
        return self.session.xenapi.VM.get_platform(self.vm)

    def set_platform(self, platform):
        return self.session.xenapi.VM.set_platform(self.vm, platform)

    def get_vCPUs(self):
        return self.session.xenapi.VM.get_VCPUs_at_startup(self.vm)

    def get_vCPU_params(self):
        return self.session.xenapi.VM.get_VCPUs_params(self.vm)

    def get_bios_strings(self):
        return self.session.xenapi.VM.get_bios_strings(self.vm)

    def set_bios_strings(self, input_bios_str):

        tmp_bios_str = self.session.xenapi.VM.get_bios_strings(self.vm)
        bios_str = {**tmp_bios_str, **input_bios_str}
        self.session.xenapi.VM.set_bios_strings(self.vm, bios_str)
        return True

    def get_memory(self):

        return self.session.xenapi.VM.get_memory_static_max(self.vm)

    async def delete(self):
        from XenGarden.VBD import VBD

        vbds = self.get_Disks()
        for vbd in vbds:
            vbd.destroy()

        task = self.session.xenapi.Async.VM.destroy(self.vm)
        await Common.xenapi_task_handler(self.session, task, True)

        return True

    async def destroy(self):
        return await self.delete()

    def get_VBDs(self, vbd_type=None):
        from XenGarden.VBD import VBD

        vbds = self.session.xenapi.VM.get_VBDs(self.vm)

        vbd_list = []
        for vbd in vbds:
            try:
                vbd_obj = VBD(self.session, vbd)
                vbd_obj.get_uuid()

                if vbd_type is not None:
                    if vbd_type == vbd_obj.get_type():
                        vbd_list.append(vbd_obj)
                else:
                    vbd_list.append(vbd_obj)
            except:
                pass

        return vbd_list

    def get_VIFs(self):
        from XenGarden.VIF import VIF

        vifs = self.session.xenapi.VM.get_VIFs(self.vm)

        vif_list = []
        for vif in vifs:
            try:
                thisVIF = VIF(self.session, vif)
                thisVIF.get_uuid()
                vif_list.append(thisVIF)
            except:
                pass

        return vif_list

    def get_VIF(self):
        vifs = self.get_VIFs()
        if vifs is not None and len(vifs) > 0:
            return vifs[0]
        else:
            return None

    def get_CD(self):
        return self.get_CDs()[0]

    def get_CDs(self):
        return self.get_VBDs("CD")

    def get_Disks(self):
        return self.get_VBDs("Disk")



from deprecated import deprecated


class Host:
    """ The Host Object """

    def __init__(self, session, host):
        self.session = session
        self.host = host

    @staticmethod
    def get_by_uuid(session, uuid):
        """ returns Host object that has specific uuid """

        host = session.xenapi.host.get_by_uuid(uuid)

        if host is not None:
            return Host(session, host)
        else:
            return None

    @staticmethod
    def get_by_name(session, label):
        """ returns Host object that has specific name """

        host = session.xenapi.host.get_by_name_label(label)

        if host is not None:
            return Host(session, host)
        else:
            return None

    @staticmethod
    def list_host(session):
        """gets Hosts available in specific XenServer
        session: the XenServer Connection Session
        Returns Available Templates (List of Host object)"""

        allHostList = []
        allHosts = session.xenapi.host.get_all()

        for hostF in allHosts:
            host = Host(session, hostF)
            allHostList.append(host)

        return allHostList

    def get_record(self):
        return self.session.xenapi.host.get_record(self.host)

    @deprecated
    def serialize(self) -> dict:

        return {
            "uuid": self.get_uuid(),
            "name": self.get_name(),
            "description": self.get_description(),
            "enabled": self.get_enabled(),
            "memory": {
                "free": self.get_free_memory(),
                "total": self.get_total_memory(),
            },
            "cpu": self.get_cpu_info(),
            "bios": self.get_bios_strings(),
            "version": self.get_software_version(),
        }

    def get_uuid(self):
        """ get UUID of Host """

        return self.session.xenapi.host.get_uuid(self.host)

    def get_free_memory(self):
        """ get Free Memory of Host """

        return self.session.xenapi.host.compute_free_memory(self.host)

    def get_total_memory(self):
        """ get Total Memory of Host """

        metrics = self.session.xenapi.host.get_metrics(self.host)
        return self.session.xenapi.host_metrics.get_memory_total(metrics)

    def get_cpu_info(self):
        """ get CPU Info of Host """

        return self.session.xenapi.host.get_cpu_info(self.host)

    def get_software_version(self):
        """ get Software Version of Host """

        return self.session.xenapi.host.get_software_version(self.host)

    def disable(self):
        """ Disable Host """

        return self.session.xenapi.host.disable(self.host)

    def enable(self):
        """ Enable Host """

        return self.session.xenapi.host.enable(self.host)

    def get_enabled(self):
        """ Get Host is Enabled """

        return self.session.xenapi.host.get_enabled(self.host)

    def evacuate(self):
        """ Evacuate Host """

        return self.session.xenapi.host.evacuate(self.host)

    def get_address(self):
        """ Get Address of Host """

        return self.session.xenapi.host.evacuate(self.host)

    def get_bios_strings(self):
        """ Get BIOS Strings of Host """

        return self.session.xenapi.host.get_bios_strings(self.host)

    def get_capabilities(self):
        """ Get capabilities of Host """

        return self.session.xenapi.host.get_capabilites(self.host)

    def get_name(self):
        """ Get name of Host """

        return self.session.xenapi.host.get_name_label(self.host)



from enum import Enum

from XenAPI.XenAPI import Failure


class VIFLockingMode(Enum):
    DEFAULT = "network_default"
    LOCKED = "locked"
    UNLOCKED = "unlocked"
    DISABLED = "disabled"


class VIFIPv4ConfigurationMode(Enum):
    NONE = "None"
    STATIC = "Static"


class VIFIPv6ConfigurationMode(Enum):
    NONE = "None"
    STATIC = "Static"


class VIF:
    """ The Virtual Interface """

    def __init__(self, session, vif):
        self.session = session
        self.vif = vif

    @staticmethod
    def get_by_uuid(session, uuid):
        """ returns VIF object that has specific uuid """

        vif = session.xenapi.VIF.get_by_uuid(uuid)

        if vif is not None:
            return VIF(session, vif)
        else:
            return None

    @staticmethod
    def get_all(session):
        """gets VIFs available in specific XenServer
        session: the XenServer Connection Session
        Returns Available Templates (List of VIF object)"""

        allVIFList = []
        allVIFs = session.xenapi.VIF.get_all()

        for vif in allVIFs:
            thisVif = VIF(session, vif)
            allVIFList.append(thisVif)

        return allVIFList

    def get_uuid(self):
        return self.session.xenapi.VIF.get_uuid(self.vif)

    def get_record(self):
        """ Returns Information of the VIF """
        return self.session.xenapi.VIF.get_record(self.vif)

    def get_attached(self):
        return self.session.xenapi.VIF.get_currently_attached(self.vif)

    def plug(self):
        self.session.xenapi.VIF.plug(self.vif)
        return True

    def unplug(self):
        self.session.xenapi.VIF.unplug(self.vif)
        return True

    def unplug_force(self):
        self.session.xenapi.VIF.unplug_force(self.vif)
        return True

    def get_locking_mode(self):
        return self.session.xenapi.VIF.get_locking_mode(self.vif)

    def set_locking_mode(self, locking_mode: VIFLockingMode):
        self.session.xenapi.VIF.set_locking_mode(self.vif, locking_mode)
        return True

    def set_locking_mode(self, locking_mode: VIFLockingMode):
        self.session.xenapi.VIF.set_locking_mode(self.vif, locking_mode)
        return True

    def lock(self, locking_mode):
        self.set_locking_mode("locked")
        return True

    def unlock(self, locking_mode):
        self.set_locking_mode("unlocked")
        return True

    def get_vm(self):
        from XenGarden.VM import VM

        try:
            vm = self.session.xenapi.VIF.get_VM(self.vif)
            vm = VM(self.session, vm)
            vm.get_uuid()

            return vm
        except Failure as xenapi_error:
            if xenapi_error.details[0] == "HANDLE_INVALID":
                return None
            else:
                raise xenapi_error

    def get_mac(self):
        mac = self.session.xenapi.VIF.get_MAC(self.vif)
        return mac

    def get_mtu(self):
        return self.session.xenapi.VIF.get_MTU(self.vif)

    def get_qos_type(self):
        return self.session.xenapi.VIF.get_qos_algorithm_type(self.vif)

    def set_qos_type(self, qos_type):
        return self.session.xenapi.VIF.get_qos_algorithm_type(self.vif, qos_type)

    def get_qos_info(self):
        return self.session.xenapi.VIF.get_qos_algorithm_params(self.vif)

    def set_qos_info(self, qos_params):
        self.session.xenapi.VIF.set_qos_algorithm_params(self.vif, qos_params)
        return True

    def supported_qos_types(self):
        return self.session.xenapi.VIF.get_qos_supported_algorithms(self.vif)

    def config_ipv4(
        self, ipv4_config_mode: VIFIPv4ConfigurationMode, ip: str, gateway: str
    ):
        self.session.xenapi.VIF.configure_ipv4(self.vif, ipv4_config_mode, ip, gateway)
        return True

    def get_address_v4(self):
        return self.session.xenapi.VIF.get_ipv4_addresses(self.vif)

    def get_gateway_v4(self):
        return self.session.xenapi.VIF.get_ipv4_gateway(self.vif)

    def get_allowed_address_v4(self):
        return self.session.xenapi.VIF.get_ipv4_allowed(self.vif)

    def set_allowed_address_v4(self, ips):
        self.session.xenapi.VIF.set_ipv4_allowed(self.vif, ips)
        return True

    def add_allowed_address_v4(self, ip):
        self.session.xenapi.VIF.add_ipv4_allowed(self.vif, ip)
        return True

    def remove_allowed_address_v4(self, ip):
        self.session.xenapi.VIF.remove_ipv4_allowed(self.vif, ip)
        return True

    def config_ipv6(
        self, ipv6_config_mode: VIFIPv6ConfigurationMode, ip: str, gateway: str
    ):
        self.session.xenapi.VIF.configure_ipv4(self.vif, ipv6_config_mode, ip, gateway)
        return True

    def get_address_v6(self):
        return self.session.xenapi.VIF.get_ipv6_addresses(self.vif)

    def get_gateway_v6(self):
        return self.session.xenapi.VIF.get_ipv6_gateway(self.vif)

    def get_allowed_address_v6(self):
        return self.session.xenapi.VIF.get_ipv6_allowed(self.vif)

    def set_allowed_address_v6(self, ips):
        self.session.xenapi.VIF.set_ipv6_allowed(self.vif, ips)
        return True

    def add_allowed_address_v6(self, ip):
        self.session.xenapi.VIF.add_ipv6_allowed(self.vif, ip)
        return True

    def remove_allowed_address_v6(self, ip):
        self.session.xenapi.VIF.remove_ipv6_allowed(self.vif, ip)
        return True



from setuptools import setup, find_packages

setup(
    name='XenGarden',
    version='1.2.5',
    description='XenGarden, A Python XenAPI Wrapper for Citrix Hypervisor and XCP-ng Servers',
    author='Stella IT',
    author_email='contact@stella-it.com',
    url='https://github.com/Stella-IT/XenGarden',
    packages=find_packages(exclude=['.vscode']),
    keywords=[
        'XenGarden', 'XenAPI', 'XenAPI Wrapper', 'Citrix Hypervisor', 'XCP-ng'
    ],
    python_requires='>=3',
    install_requires=[
        "xenapi", 
        "deprecated"
        ],
    zip_safe=False,
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ]
)



import asyncio

from deprecated import deprecated


class Console:
    """ The Virtual Console """

    def __init__(self, session, console):
        self.session = session
        self.console = console

    @deprecated
    async def serialize(self) -> dict:
        (location, protocol, uuid) = asyncio.gather(
            self.get_location(), self.get_protocol(), self.get_uuid()
        )

        return {
            "location": location,
            "protocol": protocol,
            "uuid": uuid,
        }

    @staticmethod
    def get_by_uuid(session, uuid):
        """ returns Console object that has specific uuid """

        console = session.xenapi.console.get_by_uuid(uuid)
        if console is not None:
            return Console(session, console)
        else:
            return None

    def get_record(self):
        """ Returns Information of the Console """
        return self.session.xenapi.console.get_record(self.console)

    def get_location(self):
        """ The Location for Console """

        return self.session.xenapi.console.get_location(self.console)

    def get_uuid(self):
        """ Returns UUID of Console """

        return self.session.xenapi.console.get_uuid(self.console)

    def get_protocol(self):
        """ Returns Protocol of Console """
        return self.session.xenapi.console.get_protocol(self.console)

    def get_VM(self):
        """ Returns which VM is this console attached to """

        return self.session.xenapi.Async.console.get_VM(self.console)



from deprecated import deprecated

from XenGarden.VDI import VDI


class SR:
    """ The Storage Repository Object """

    def __init__(self, session, sr):
        self.session = session
        self.sr = sr

    @staticmethod
    def get_all(session):
        """ get all SR object available """

        srs = session.xenapi.SR.get_all()

        sr_list = []
        for sr in srs:
            sr_list.append(SR(session, sr))

        return sr_list

    @staticmethod
    def get_by_name(session, name):
        """ returns SR object that has specific name"""

        srs = session.xenapi.SR.get_by_name_label(name)

        srs_list = []
        for sr in srs:
            srs_list.append(SR(session, sr))

        return srs_list

    @staticmethod
    def get_by_uuid(session, uuid):
        """ returns SR object that has specific uuid """

        sr = session.xenapi.SR.get_by_uuid(uuid)

        if sr is not None:
            return SR(session, sr)
        else:
            return None

    def get_record(self):
        return self.session.xenapi.SR.get_record(self.sr)

    @deprecated
    def serialize(self) -> dict:
        """ Returns Info of this SR """
        vdis = self.get_VDIs()
        vdi_list = []

        if vdis is not None:
            for vdi in vdis:
                vdi_list.append(vdi.serialize())
        else:
            vdi_list = None

        return {
            "name": self.get_name(),
            "description": self.get_description(),
            "size": self.get_physical_size(),
            "uuid": self.get_uuid(),
            "content_type": self.get_content_type(),
            "type": self.get_type(),
            "vdis": vdi_list,
        }

    def get_VDIs(self):

        vdis = self.session.xenapi.SR.get_VDIs(self.sr)

        vdi_list = []
        for vdi in vdis:
            vdi_list.append(VDI(self.session, vdi))

        return vdi_list

    def get_name(self):

        return self.session.xenapi.SR.get_name_label(self.sr)

    def get_description(self):

        return self.session.xenapi.SR.get_name_description(self.sr)

    def get_physical_size(self):

        return self.session.xenapi.SR.get_physical_size(self.sr)

    def get_physical_utilisation(self):

        return self.session.xenapi.SR.get_physical_utilisation(self.sr)

    def get_uuid(self):

        return self.session.xenapi.SR.get_uuid(self.sr)

    def get_content_type(self):

        return self.session.xenapi.SR.get_content_type(self.sr)

    def get_type(self):

        return self.session.xenapi.SR.get_type(self.sr)

    def scan(self):

        self.session.xenapi.SR.scan(self.sr)



from deprecated import deprecated

from XenGarden.Common import Common


class VDI:
    """ The Virtual Disk Image Object """

    def __init__(self, session, vdi):
        self.session = session
        self.vdi = vdi

    @staticmethod
    def get_by_name(session, name):
        """ returns SR object that has specific name """

        vdis = session.xenapi.VDI.get_by_name_label(name)

        vdi_list = []
        for vdi in vdis:
            vdi_list.append(VDI(session, vdi))

        return vdi_list

    @staticmethod
    def get_by_uuid(session, uuid):
        """ returns SR object that has specific uuid """

        vdi = session.xenapi.VDI.get_by_uuid(uuid)

        if vdi is not None:
            return VDI(session, vdi)
        else:
            return None

    @staticmethod
    def get_all(session):
        """ returns SR object that exists on _host """

        vdis = session.xenapi.VDI.get_all()

        vdi_list = []
        for vdi in vdis:
            vdi_list.append(VDI(session, vdi))

        return vdi_list

    def get_record(self):
        return self.session.xenapi.VDI.get_record(self.vdi)

    @deprecated
    def serialize(self) -> dict:
        return {
            "name": self.get_name(),
            "description": self.get_description(),
            "uuid": self.get_uuid(),
            "location": self.get_location(),
            "type": self.get_type(),
        }

    def get_name(self):

        return self.session.xenapi.VDI.get_name_label(self.vdi)

    def get_description(self):

        return self.session.xenapi.VDI.get_name_description(self.vdi)

    def get_uuid(self):

        return self.session.xenapi.VDI.get_uuid(self.vdi)

    def get_type(self):

        return self.session.xenapi.VDI.get_type(self.vdi)

    def get_location(self):

        return self.session.xenapi.VDI.get_location(self.vdi)

    def get_virtual_size(self) -> str:

        return self.session.xenapi.VDI.get_virtual_size(self.vdi)

    def destroy(self):

        return self.session.xenapi.VDI.destroy(self.vdi)

    def get_SR(self):
        from XenGarden.SR import SR

        data = self.session.xenapi.VDI.get_SR(self.vdi)
        sr = SR(self.session, data)

        return sr

    async def clone(self):

        task = self.session.xenapi.Async.VDI.clone(self.vdi)
        data = await Common.xenapi_task_handler(self.session, task, True)

        vdi = VDI(self.session, data)
        return vdi

    async def copy(self, sr):

        task = self.session.xenapi.Async.VDI.copy(self.vdi, sr.sr)
        data = await Common.xenapi_task_handler(self.session, task, True)

        vdi = VDI(self.session, data)
        return vdi

    async def resize(self, size: str):

        task = self.session.xenapi.Async.VDI.resize(self.vdi, size)
        await Common.xenapi_task_handler(self.session, task, True)

        return True
